/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */
package org.olat.ims.qti.statistics.model;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.annotations.GenericGenerator;
import org.olat.core.id.CreateInfo;
import org.olat.core.id.Persistable;

/**
 * 
 * This is a specialized mapping
 * 
 * @author srosse, stephane.rosse@frentix.com, http://www.frentix.com
 *
 */
@Entity(name="qtistatsresultset")
@Table(name="o_qtiresultset")
public class QTIStatisticResultSet implements CreateInfo, Persistable {

	private static final long serialVersionUID = -3961571038651555893L;
	
	@Id
	@GeneratedValue(generator = "system-uuid")
	@GenericGenerator(name = "system-uuid", strategy = "hilo")
	@Column(name="resultset_id", nullable=false, unique=true, insertable=true, updatable=false)
	private Long key;
	@Temporal(TemporalType.TIMESTAMP)
	@Column(name="creationdate", nullable=false, insertable=true, updatable=false)
	private Date creationDate;
	@Temporal(TemporalType.TIMESTAMP)
	@Column(name="lastmodified", nullable=false, insertable=true, updatable=false)
	private Date lastModified;

	@Column(name="olatresource_fk", nullable=false, insertable=true, updatable=false)
	private Long olatResource;
	@Column(name="olatresourcedetail", nullable=false, insertable=true, updatable=false)
	private String olatResourceDetail;
	@Column(name="identity_id", nullable=false, insertable=true, updatable=false)
	private Long identityKey;
	@Column(name="ispassed", nullable=false, insertable=true, updatable=false)
	private Boolean isPassed;
	@Column(name="score", nullable=false, insertable=true, updatable=false)
	private float score;
	@Column(name="duration", nullable=false, insertable=true, updatable=false)
	private Long duration;

	public Long getKey() {
		return key;
	}

	public void setKey(Long key) {
		this.key = key;
	}

	public Date getCreationDate() {
		return creationDate;
	}
	
	public void setCreationDate(Date creationDate) {
		this.creationDate = creationDate;
	}

	public Date getLastModified() {
		return lastModified;
	}

	public void setLastModified(Date date) {
		lastModified = date;
	}

	public Long getIdentityKey() {
		return identityKey;
	}
	
	public void setIdentityKey(Long identityKey) {
		this.identityKey = identityKey;
	}


	public Long getOlatResource() {
		return olatResource;
	}

	public void setOlatResource(Long olatResource) {
		this.olatResource = olatResource;
	}

	public String getOlatResourceDetail() {
		return olatResourceDetail;
	}

	public void setOlatResourceDetail(String olatResourceDetail) {
		this.olatResourceDetail = olatResourceDetail;
	}

	public Boolean getIsPassed() {
		return isPassed;
	}

	public void setIsPassed(Boolean isPassed) {
		this.isPassed = isPassed;
	}

	public float getScore() {
		return score;
	}

	public void setScore(float score) {
		this.score = score;
	}

	public Long getDuration() {
		return duration;
	}

	public void setDuration(Long duration) {
		this.duration = duration;
	}

	@Override
	public int hashCode() {
		return getKey() == null ? 87221 : getKey().hashCode();
	}

	@Override
	public boolean equalsByPersistableKey(Persistable persistable) {
		return equals(persistable);
	}

	@Override
	public boolean equals(Object obj) {
		if(this == obj) {
			return true;
		}
		if(obj instanceof QTIStatisticResultSet) {
			QTIStatisticResultSet set = (QTIStatisticResultSet)obj;
			return getKey() != null && getKey().equals(set.getKey());
		}
		return false;
	}
}

